/***
 * E-buttons ver 0.2 - A small and very simple Epplet
 * kevin mulholland, <kevin@moodfarm.demon.co.uk>
 * There ain't no license for this cos its trivial, use it,
 * abuse it, sell it the choice is yours, only respect the
 * fact that I wrote it
 * v0.2 11 Jan 2000, mostly changed the makefile, but also added a bit of code
 * to determine if epplet is mostly horizontal or vertical so this can be
 * passed to Epplet_Init
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <epplet.h>

#define VERSION			"0.2"

/* the number of buttons available on each axis */
#define X_MAX				3
#define Y_MAX				3
#define MAX_BUTTONS	X_MAX * Y_MAX
#define	MAX_CMD_LEN	255

char commands[Y_MAX * X_MAX][MAX_CMD_LEN+1] ;

static void button_cb(void *data);
static void load_conf(void) ;

/* these are just any old things that you may have on your system, many
 * commands such as this come with defaults that only apply to the authors
 * system, and hence often do not work on your system, so I guess most of the
 * buttons should do things if you press them
 */
char *default_commands[] =
{
	"gimp",
	"xterm -font 10x20",
	"xedit",
	"xbanner -label 'E-buttons'",
	"xscreensaver-command -activate",
	"xeyes",
	"eterm",
	"xsysingo",
	"rxvt"
} ;
#define MAX_DEFAULTS sizeof( default_commands) / sizeof( char *)

/* ------------------------------------------------------------------------- */

/*** button_cb
 * the button callback function
 * @param data, the string value of the button pressed
 */
static void button_cb(void *data)
{
	int cmd_num = atoi(data) - 1 ;

	printf( "running command [%s]\n", commands[cmd_num]) ;

	/* now run the required program */
	Epplet_spawn_command(commands[cmd_num]);
}

/*** load_conf
 * load the button programs, this will create a default config file
 * if required
 */
static void load_conf(void)
{
  int i, save = 0;
  char key[20], *value;
  char default_cmd[ MAX_CMD_LEN+1] ;

  for( i = 0 ; i < MAX_BUTTONS ; i++)
  {
  	sprintf( key, "%d", i+1) ;
  	value = Epplet_query_config( key) ;
  	
  	/* if there is no config for this button, create a default */
  	if( value == NULL)
  	{
  		/* check in range of defaults, if outside range then the last default
  		 * will be used
  		 */
  		if( i < MAX_DEFAULTS)
  		{
  			value = default_commands[i] ;
  		}
  		else
  		{
  			sprintf( default_cmd, "xmessage this is default action %s", key) ;
	  		value = default_cmd ;
  		}
	  	Epplet_add_config( key, value) ;
  		save = 1 ;
  	}
 		/* make sure prog does not exceed max allowed length */
 		if( strlen( value) > MAX_CMD_LEN)
 		{
	 		value[MAX_CMD_LEN] = 0 ;	
 		}

	  strcpy( commands[i], value) ;
  }
  /* in case loaded defaults */
  if( save)
	  Epplet_save_config() ;
}

int main(int argc, char **argv)
{
	int x, y;
	int vert = X_MAX >= Y_MAX ? 0 : 1 ;				/* decide orientation of epplet */
	char button_str[Y_MAX][X_MAX][5] ;
	
	Epplet_Init("E-buttons", VERSION, "E buttons epplet", X_MAX, Y_MAX,
							argc, argv, vert);

	/* allow us to use a config file */
  Epplet_load_config();
  /* now load the config details */
  load_conf() ;
	
	for( y = 0 ; y < Y_MAX ; y++)
	{
		for( x = 0 ; x < X_MAX ; x++)
		{
			sprintf( button_str[y][x], "%d", 1 + (y * X_MAX) + x) ;
			Epplet_gadget_show(	Epplet_create_text_button(button_str[y][x],
													2+ (x*15), 1+(y*15), 14, 14,
													(void*)button_cb, button_str[y][x]) );
		}
	}

	Epplet_show();
	Epplet_Loop();

	return 0;
}
